Package: stepR
Title: Multiscale Change-Point Inference
Version: 2.1-11
Authors@R: c(person("Pein", "Florian", role = c("aut", "cre"), email = "f.pein@lancaster.ac.uk"),
    person("Thomas", "Hotz", role = "aut", email = "thomas.hotz@tu-ilmenau.de"),
    person("Hannes", "Sieling", role = "aut", email = "hsielin@uni-goettingen.de"),
    person("Timo", "Aspelmeier", role = "ctb", email = "timo.aspelmeier@mathematik.uni-goettingen.de"))
Depends: R (>= 3.3.0)
Imports: Rcpp (>= 0.12.3), lowpassFilter (>= 1.0.0), R.cache (>=
        0.10.0), digest (>= 0.6.10), stats, graphics, methods
LinkingTo: Rcpp
Suggests: testthat (>= 1.0.0), knitr
VignetteBuilder: knitr
Description: Allows fitting of step-functions to univariate serial data where neither the number of jumps nor their positions is known by implementing the multiscale regression estimators SMUCE, simulataneous multiscale changepoint estimator, (K. Frick, A. Munk and H. Sieling, 2014) <doi:10.1111/rssb.12047> and HSMUCE, heterogeneous SMUCE, (F. Pein, H. Sieling and A. Munk, 2017) <doi:10.1111/rssb.12202>. In addition, confidence intervals for the change-point locations and bands for the unknown signal can be obtained.
License: GPL-3
Classification/MSC: 62G08, 92C40, 92D20
LazyData: yes
NeedsCompilation: yes
Packaged: 2026-03-23 17:03:51 UTC; pein
Author: Pein Florian [aut, cre],
  Thomas Hotz [aut],
  Hannes Sieling [aut],
  Timo Aspelmeier [ctb]
Maintainer: Pein Florian <f.pein@lancaster.ac.uk>
Repository: CRAN
Date/Publication: 2026-03-24 06:10:18 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 06:29:42 UTC; unix
Archs: stepR.so.dSYM
